@@ -447,7 +447,7 @@ extend_iter(bitarrayobject *self, PyObject *iter)
     PyObject *item;
 
     assert(PyIter_Check(iter));
-    while ((item = PyIter_Next(iter)) != NULL) {
+    while ((item = PyIter_Next(iter))) {
         if (append_item(self, item) < 0) {
             Py_DECREF(item);
             return -1;
@@ -485,41 +485,62 @@ extend_list(bitarrayobject *self, PyObject *list)
 }
 
 static int
-extend_01(bitarrayobject *self, PyObject *bytes)
+extend_bytes01(bitarrayobject *self, PyObject *bytes)
 {
-    Py_ssize_t nbytes, i;
-    char c, *data;
-    int vi = 0;
+    const Py_ssize_t original_nbits = self->nbits;
+    unsigned char c;
+    char *data;
+    int vi = 0;  /* to avoid uninitialized warning for some compilers */
 
     assert(PyBytes_Check(bytes));
-    nbytes = PyBytes_GET_SIZE(bytes);
-    if (nbytes == 0)
-        return 0;
+    data = PyBytes_AS_STRING(bytes);
 
-    if (resize(self, self->nbits + nbytes) < 0)
-        return -1;
-
-    data = PyBytes_AsString(bytes);
-    for (i = 0; i < nbytes; i++) {
-        c = data[i];
+    while ((c = *data++)) {
         switch (c) {
         case '0': vi = 0; break;
         case '1': vi = 1; break;
+        case ' ':
+        case '\n':
+        case '\r':
+        case '\t':
+        case '\v':
+            continue;
         default:
-            PyErr_Format(PyExc_ValueError,
-                         "character must be '0' or '1', got '%c'", c);
+            /* ensure no bits are added on error */
+            if (resize(self, original_nbits) < 0)
+                return -1;
+            PyErr_Format(PyExc_ValueError, "expected '0' or '1' "
+                         "(or whitespace), got '%c' (0x%02x)", c, c);
             return -1;
         }
-        setbit(self, self->nbits - nbytes + i, vi);
+        if (resize(self, self->nbits + 1) < 0)
+            return -1;
+        setbit(self, self->nbits - 1, vi);
     }
     return 0;
 }
 
+static int
+extend_unicode01(bitarrayobject *self, PyObject *unicode)
+{
+    PyObject *bytes;
+    int res;
+
+    assert(PyUnicode_Check(unicode));
+    bytes = PyUnicode_AsASCIIString(unicode);
+    if (bytes == NULL)
+        return -1;
+
+    assert(PyBytes_Check(bytes));
+    res = extend_bytes01(self, bytes);
+    Py_DECREF(bytes);  /* drop bytes */
+    return res;
+}
+
 static int
 extend_dispatch(bitarrayobject *self, PyObject *obj)
 {
     PyObject *iter;
-    int res;
 
     /* dispatch on type */
     if (bitarray_Check(obj))                              /* bitarray */
@@ -535,21 +556,12 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
                         "use .pack() or .frombytes() instead");
         return -1;
 #else
-        return extend_01(self, obj);
+        return extend_bytes01(self, obj);
 #endif
     }
 
-    if (PyUnicode_Check(obj)) {                /* (unicode) string 01 */
-        PyObject *bytes;
-
-        bytes = PyUnicode_AsEncodedString(obj, NULL, NULL);
-        if (bytes == NULL)
-            return -1;
-        assert(PyBytes_Check(bytes));
-        res = extend_01(self, bytes);
-        Py_DECREF(bytes);  /* drop bytes */
-        return res;
-    }
+    if (PyUnicode_Check(obj))                           /* unicode 01 */
+        return extend_unicode01(self, obj);
 
     if (PyIter_Check(obj))                                    /* iter */
         return extend_iter(self, obj);
@@ -557,6 +569,8 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
     /* finally, try to get the iterator of the object */
     iter = PyObject_GetIter(obj);
     if (iter) {
+        int res;
+
         res = extend_iter(self, iter);
         Py_DECREF(iter);
         return res;
@@ -768,7 +782,7 @@ bitarray_search(bitarrayobject *self, PyObject *args)
     PyObject *item = NULL, *x;
     Py_ssize_t limit = -1;
     bitarrayobject *xa;
-    Py_ssize_t p;
+    Py_ssize_t p = 0;
 
     if (!PyArg_ParseTuple(args, "O|n:search", &x, &limit))
         return NULL;
@@ -788,13 +802,8 @@ bitarray_search(bitarrayobject *self, PyObject *args)
     if (xa->nbits > self->nbits || limit == 0)
         return list;
 
-    p = 0;
-    while (1) {
-        p = search(self, xa, p);
-        if (p < 0)
-            break;
-        item = PyLong_FromSsize_t(p);
-        p++;
+    while ((p = search(self, xa, p)) >= 0) {
+        item = PyLong_FromSsize_t(p++);
         if (item == NULL || PyList_Append(list, item) < 0) {
             Py_XDECREF(item);
             Py_XDECREF(list);
@@ -904,14 +913,14 @@ bitarray_reduce(bitarrayobject *self)
         dict = Py_None;
         Py_INCREF(dict);
     }
-    /* the first byte indicates the number of unused bits at the end, and
-       the rest of the bytes consist of the raw binary data */
     data = (char *) PyMem_Malloc(nbytes + 1);
     if (data == NULL) {
         PyErr_NoMemory();
         goto error;
     }
-    data[0] = (char) setunused(self);
+    /* first byte contains the number of unused bits */
+    *data = (char) setunused(self);
+    /* remaining bytes contain buffer */
     memcpy(data + 1, self->ob_item, (size_t) nbytes);
     repr = PyBytes_FromStringAndSize(data, nbytes + 1);
     if (repr == NULL)
@@ -928,6 +937,37 @@ bitarray_reduce(bitarrayobject *self)
 PyDoc_STRVAR(reduce_doc, "state information for pickling");
 
 
+/* The head byte % 8 specifies the number of unused bits (in last buffer
+   byte), the remaining bytes consist of the buffer itself */
+static PyObject *
+unpickle(PyTypeObject *type, PyObject *bytes, int endian)
+{
+    PyObject *res;
+    Py_ssize_t nbytes;
+    unsigned char head;
+    char *data;
+
+    assert(PyBytes_Check(bytes));
+    nbytes = PyBytes_GET_SIZE(bytes);
+    assert(nbytes > 0);
+    data = PyBytes_AS_STRING(bytes);
+    head = *data;
+
+    if (nbytes == 1 && head % 8) {
+        PyErr_Format(PyExc_ValueError,
+                     "invalid header byte 0x%02x", head);
+        return NULL;
+    }
+    res = newbitarrayobject(type,
+                            BITS(nbytes - 1) - ((Py_ssize_t) (head % 8)),
+                            endian);
+    if (res == NULL)
+        return NULL;
+    memcpy(((bitarrayobject *) res)->ob_item, data + 1, (size_t) nbytes - 1);
+    return res;
+}
+
+
 static PyObject *
 bitarray_reverse(bitarrayobject *self)
 {
@@ -1161,7 +1201,7 @@ bitarray_frombytes(bitarrayobject *self, PyObject *bytes)
         return NULL;
 
     memcpy(self->ob_item + (Py_SIZE(self) - nbytes),
-           PyBytes_AsString(bytes), (size_t) nbytes);
+           PyBytes_AS_STRING(bytes), (size_t) nbytes);
 
     if (delete_n(self, t, p) < 0)
         return NULL;
@@ -1321,7 +1361,7 @@ bitarray_pack(bitarrayobject *self, PyObject *bytes)
     if (resize(self, self->nbits + nbytes) < 0)
         return NULL;
 
-    data = PyBytes_AsString(bytes);
+    data = PyBytes_AS_STRING(bytes);
     for (i = 0; i < nbytes; i++)
         setbit(self, self->nbits - nbytes + i, data[i] ? 1 : 0);
 
@@ -2016,7 +2056,7 @@ bitarray_encode(bitarrayobject *self, PyObject *args)
         return NULL;
     }
     /* extend self with the bitarrays from codedict */
-    while ((symbol = PyIter_Next(iter)) != NULL) {
+    while ((symbol = PyIter_Next(iter))) {
         value = PyDict_GetItem(codedict, symbol);
         Py_DECREF(symbol);
         if (value == NULL) {
@@ -2198,7 +2238,8 @@ binode_to_dict(binode *nd, PyObject *dict, bitarrayobject *prefix)
         t = (bitarrayobject *) bitarray_copy(prefix);
         if (t == NULL)
             return -1;
-        resize(t, t->nbits + 1);
+        if (resize(t, t->nbits + 1) < 0)
+            return -1;
         setbit(t, t->nbits - 1, k);
         ret = binode_to_dict(nd->child[k], dict, t);
         Py_DECREF((PyObject *) t);
@@ -2772,6 +2813,23 @@ endian_from_string(const char* string)
     return -1;
 }
 
+static PyObject*
+bitarray_from_index(PyTypeObject *type, PyObject *index, int endian)
+{
+    Py_ssize_t nbits;
+
+    assert(PyIndex_Check(index));
+    nbits = PyNumber_AsSsize_t(index, PyExc_IndexError);
+    if (nbits == -1 && PyErr_Occurred())
+        return NULL;
+
+    if (nbits < 0) {
+        PyErr_SetString(PyExc_ValueError,
+                        "cannot create bitarray with negative length");
+        return NULL;
+    }
+    return newbitarrayobject(type, nbits, endian);
+}
 
 static PyObject *
 bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
@@ -2801,64 +2859,23 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     }
 
     /* index (a number) */
-    if (PyIndex_Check(initial)) {
-        Py_ssize_t nbits;
-
-        nbits = PyNumber_AsSsize_t(initial, PyExc_IndexError);
-        if (nbits == -1 && PyErr_Occurred())
-            return NULL;
-
-        if (nbits < 0) {
-            PyErr_SetString(PyExc_ValueError,
-                            "cannot create bitarray with negative length");
-            return NULL;
-        }
-        return newbitarrayobject(type, nbits, endian);
-    }
-
-    /* from bitarray itself */
-    if (bitarray_Check(initial)) {
-#define np  ((bitarrayobject *) initial)
-        res = newbitarrayobject(type, np->nbits,
-                                endian_str == NULL ? np->endian : endian);
-        if (res == NULL)
-            return NULL;
-        memcpy(((bitarrayobject *) res)->ob_item, np->ob_item,
-               (size_t) Py_SIZE(np));
-#undef np
-        return res;
-    }
+    if (PyIndex_Check(initial))
+        return bitarray_from_index(type, initial, endian);
 
     /* bytes (for pickling) */
-    if (PyBytes_Check(initial)) {
-        Py_ssize_t nbytes;
-        char *data;
+    if (PyBytes_Check(initial) && PyBytes_GET_SIZE(initial) > 0) {
+        unsigned char head = *PyBytes_AS_STRING(initial);
 
-        nbytes = PyBytes_Size(initial);
-        if (nbytes == 0)        /* no bytes */
-            return newbitarrayobject(type, 0, endian);
-
-        data = PyBytes_AsString(initial);
-        if (0 <= data[0] && data[0] < 8) {
-            /* when the first character is smaller than 8, it indicates the
-               number of unused bits at the end, and rest of the bytes
-               consist of the raw binary data */
-            if (nbytes == 1 && data[0] > 0) {
-                PyErr_Format(PyExc_ValueError,
-                             "did not expect 0x0%d", (int) data[0]);
-                return NULL;
-            }
-            res = newbitarrayobject(type,
-                                    BITS(nbytes - 1) - ((Py_ssize_t) data[0]),
-                                    endian);
-            if (res == NULL)
-                return NULL;
-            memcpy(((bitarrayobject *) res)->ob_item, data + 1,
-                   (size_t) nbytes - 1);
-            return res;
+        if (head < 32 && head % 16 < 8) {
+            if (endian_str == NULL)  /* no endianness given in argument */
+                endian = head / 16 ? ENDIAN_BIG : ENDIAN_LITTLE;
+            return unpickle(type, initial, endian);
         }
     }
 
+    if (bitarray_Check(initial) && endian_str == NULL)
+        endian = ((bitarrayobject *) initial)->endian;
+
     /* leave remaining type dispatch to the extend method */
     res = newbitarrayobject(type, 0, endian);
     if (res == NULL)
