@@ -5,6 +5,8 @@ from __future__ import absolute_import
 
 import os
 import sys
+import shutil
+import tempfile
 import unittest
 from string import hexdigits
 from random import choice, randint, random
@@ -14,12 +16,16 @@ from bitarray import (bitarray, frozenbitarray, bits2bytes, decodetree,
                       get_default_endian, _set_default_endian)
 from bitarray.test_bitarray import Util
 
-from bitarray.util import (zeros, urandom, make_endian, rindex, strip,
+from bitarray.util import (zeros, urandom, pprint, make_endian, rindex, strip,
                            count_n, count_and, count_or, count_xor, subset,
-                           ba2hex, hex2ba, ba2int, int2ba, huffman_code)
+                           ba2hex, hex2ba, ba2int, int2ba,
+                           serialize, deserialize, huffman_code)
 
 if sys.version_info[0] == 3:
     unicode = str
+    from io import StringIO
+else:
+    from io import BytesIO as StringIO
 
 tests = []
 
@@ -102,6 +108,65 @@ tests.append(TestsRandom)
 
 # ---------------------------------------------------------------------------
 
+class TestsPPrint(unittest.TestCase):
+
+    @staticmethod
+    def get_code_string(a):
+        f = StringIO()
+        pprint(a, stream=f)
+        return f.getvalue()
+
+    def round_trip(self, a):
+        b = eval(self.get_code_string(a))
+        self.assertEqual(b, a)
+        self.assertEqual(type(b), type(a))
+
+    def test_bitarray(self):
+        a = bitarray('110')
+        self.assertEqual(self.get_code_string(a), "bitarray('110')\n")
+        self.round_trip(a)
+
+    def test_frozenbitarray(self):
+        a = frozenbitarray('01')
+        self.assertEqual(self.get_code_string(a), "frozenbitarray('01')\n")
+        self.round_trip(a)
+
+    def test_fallback(self):
+        for a in None, 'asd', [1, 2], bitarray(), frozenbitarray('1'):
+            self.round_trip(a)
+
+    def test_subclass(self):
+        class Foo(bitarray):
+            pass
+
+        a = Foo()
+        code = self.get_code_string(a)
+        self.assertEqual(code, "Foo()\n")
+        b = eval(code)
+        self.assertEqual(b, a)
+        self.assertEqual(type(b), type(a))
+
+    def test_random(self):
+        for n in range(300):
+            self.round_trip(urandom(n))
+
+    def test_file(self):
+        tmpdir = tempfile.mkdtemp()
+        tmpfile = os.path.join(tmpdir, 'testfile')
+        a = bitarray(1000)
+        try:
+            with open(tmpfile, 'w') as fo:
+                pprint(a, fo)
+            b = eval(open(tmpfile).read())
+            self.assertEqual(a, b)
+        finally:
+            shutil.rmtree(tmpdir)
+
+
+tests.append(TestsPPrint)
+
+# ---------------------------------------------------------------------------
+
 class TestsMakeEndian(unittest.TestCase, Util):
 
     def test_simple(self):
@@ -763,6 +828,62 @@ tests.append(TestsIntegerization)
 
 # ---------------------------------------------------------------------------
 
+class TestsSerialization(unittest.TestCase, Util):
+
+    def test_explicit(self):
+        for a, s in [
+                (bitarray(0, 'little'),   b'\x00'),
+                (bitarray(0, 'big'),      b'\x10'),
+                (bitarray('1', 'little'), b'\x07\x01'),
+                (bitarray('1', 'big'),    b'\x17\x80'),
+                (bitarray('11110000', 'little'), b'\x00\x0f'),
+                (bitarray('11110000', 'big'),    b'\x10\xf0'),
+        ]:
+            self.assertEqual(serialize(a), s)
+            self.assertEQUAL(deserialize(s), a)
+
+    def test_zeros_and_ones(self):
+        for endian in 'little', 'big':
+            for n in range(100):
+                a = zeros(n, endian)
+                s = serialize(a)
+                self.assertEqual(s[1:], b'\0' * bits2bytes(n))
+                self.assertEQUAL(a, deserialize(s))
+                a.setall(1)
+                self.assertEQUAL(a, deserialize(serialize(a)))
+
+    def test_wrong_args(self):
+        self.assertRaises(TypeError, serialize, '0')
+        self.assertRaises(TypeError, serialize, bitarray(), 1)
+        self.assertRaises(TypeError, deserialize, unicode('01'))
+        self.assertRaises(TypeError, deserialize, b'\x00', 1)
+
+    def test_invalid_bytes(self):
+        self.assertRaises(ValueError, deserialize, b'')
+
+        for i in range(256):
+            b1 = bytes(bytearray([i]))
+            if i == 0 or i == 16:
+                self.assertEqual(deserialize(b1), bitarray())
+            else:
+                self.assertRaises(ValueError, deserialize, b1)
+
+            b2 = bytes(bytearray([i, 0]))
+            if i < 32 and i % 16 < 8:
+                self.assertEqual(deserialize(b2), zeros(8 - i % 8))
+            else:
+                self.assertRaises(ValueError, deserialize, b2)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            b = deserialize(serialize(a))
+            self.assertEQUAL(a, b)
+
+
+tests.append(TestsSerialization)
+
+# ---------------------------------------------------------------------------
+
 class TestsHuffman(unittest.TestCase):
 
     def test_simple(self):
