@@ -15,12 +15,13 @@ from bitarray import bitarray, bits2bytes, get_default_endian
 
 from bitarray._util import (count_n, rindex,
                             count_and, count_or, count_xor, subset,
-                            _swap_hilo_bytes, _set_bato)
+                            serialize, _swap_hilo_bytes, _set_bato)
 
 
-__all__ = ['zeros', 'urandom', 'make_endian', 'rindex', 'strip',
+__all__ = ['zeros', 'urandom', 'pprint', 'make_endian', 'rindex', 'strip',
            'count_n', 'count_and', 'count_or', 'count_xor', 'subset',
-           'ba2hex', 'hex2ba', 'ba2int', 'int2ba', 'huffman_code']
+           'ba2hex', 'hex2ba', 'ba2int', 'int2ba',
+           'serialize', 'deserialize', 'huffman_code']
 
 
 # tell the _util extension what the bitarray type object is, such that it
@@ -37,7 +38,7 @@ Create a bitarray of length, with all values 0, and optional
 endianness, which may be 'big', 'little'.
 """
     if not isinstance(length, (int, long) if _is_py2 else int):
-        raise TypeError("int expected")
+        raise TypeError("int expected, got '%s'" % type(length).__name__)
 
     a = bitarray(length, get_default_endian() if endian is None else endian)
     a.setall(0)
@@ -55,30 +56,61 @@ Return a bitarray of `length` random bits (uses `os.urandom`).
     return a
 
 
+def pprint(a, stream=None):
+    """pprint(bitarray, /, stream=None)
+
+Prints the formatted representation of object on `stream`, followed by a
+newline.  If `stream` is `None`, `sys.stdout` is used.  Elements are grouped
+in bytes (8 elements), and 8 bytes (64 elements) per line.  Non-bitarray
+objects are prinited by the standard library function `pprint.pprint()`.
+"""
+    if stream is None:
+        stream = sys.stdout
+
+    if not isinstance(a, bitarray):
+        import pprint as _pprint
+        _pprint.pprint(a, stream=stream)
+        return
+
+    multiline = bool(len(a) > 56)
+    if multiline:
+        quotes = "'''"
+    elif len(a) > 0:
+        quotes = "'"
+    else:
+        quotes = ""
+
+    stream.write("%s(%s" % (type(a).__name__, quotes))
+    for i, b in enumerate(a):
+        if multiline and i % 64 == 0:
+            stream.write('\n    ')
+        if i % 8 == 0 and i % 64 != 0:
+            stream.write(' ')
+        stream.write(str(int(b)))
+
+    if multiline:
+        stream.write('\n')
+
+    stream.write("%s)\n" % quotes)
+    stream.flush()
+
+
 def make_endian(a, endian):
     """make_endian(bitarray, endian, /) -> bitarray
 
 When the endianness of the given bitarray is different from `endian`,
 return a new bitarray, with endianness `endian` and the same elements
-as the original bitarray, i.e. even though the binary representation of the
-new bitarray will be different, the returned bitarray will equal the original
-one.
+as the original bitarray.
 Otherwise (endianness is already `endian`) the original bitarray is returned
 unchanged.
 """
     if not isinstance(a, bitarray):
-        raise TypeError("bitarray expected")
+        raise TypeError("bitarray expected, got '%s'" % type(a).__name__)
 
     if a.endian() == endian:
         return a
 
-    b = bitarray(a, endian)
-    b.bytereverse()
-    if len(a) % 8:
-        # copy last few bits directly
-        p = 8 * (bits2bytes(len(a)) - 1)
-        b[p:] = a[p:]
-    return b
+    return bitarray(a, endian)
 
 
 def strip(a, mode='right'):
@@ -88,7 +120,7 @@ Strip zeros from left, right or both ends.
 Allowed values for mode are the strings: `left`, `right`, `both`
 """
     if not isinstance(a, bitarray):
-        raise TypeError("bitarray expected")
+        raise TypeError("bitarray expected, got '%s'" % type(a).__name__)
     if not isinstance(mode, str):
         raise TypeError("str expected for mode")
     if mode not in ('left', 'right', 'both'):
@@ -118,7 +150,7 @@ Return a string containing with hexadecimal representation of
 the bitarray (which has to be multiple of 4 in length).
 """
     if not isinstance(a, bitarray):
-        raise TypeError("bitarray expected")
+        raise TypeError("bitarray expected, got '%s'" % type(a).__name__)
 
     if len(a) % 4:
         raise ValueError("bitarray length not multiple of 4")
@@ -140,7 +172,7 @@ Bitarray of hexadecimal representation.
 hexstr may contain any number of hex digits (upper or lower case).
 """
     if not isinstance(s, (str, unicode if _is_py2 else bytes)):
-        raise TypeError("str expected, got: %r" % s)
+        raise TypeError("str expected, got: '%s'" % type(s).__name__)
 
     strlen = len(s)
     if strlen % 2:
@@ -165,7 +197,7 @@ The bit-endianness of the bitarray is respected.
 `signed` indicates whether two's complement is used to represent the integer.
 """
     if not isinstance(a, bitarray):
-        raise TypeError("bitarray expected")
+        raise TypeError("bitarray expected, got '%s'" % type(a).__name__)
     length = len(a)
     if length == 0:
         raise ValueError("non-empty bitarray expected")
@@ -205,7 +237,7 @@ If signed is False and a negative integer is given, an OverflowError
 is raised.
 """
     if not isinstance(i, (int, long) if _is_py2 else int):
-        raise TypeError("int expected")
+        raise TypeError("int expected, got '%s'" % type(i).__name__)
     if length is not None:
         if not isinstance(length, int):
             raise TypeError("int expected for length")
@@ -253,6 +285,22 @@ is raised.
     return a
 
 
+def deserialize(b):
+    """deserialize(bytes, /) -> bitarray
+
+Return a bitarray given the bytes representation returned by `serialize()`.
+"""
+    if not isinstance(b, bytes):
+        raise TypeError("bytes expected, got: '%s'" % type(b).__name__)
+    if len(b) == 0:
+        raise ValueError("non-empty bytes expected")
+    head = ord(b[0]) if _is_py2 else b[0]
+    if head >= 32 or head % 16 >= 8:
+        raise ValueError('invalid header byte 0x%02x' % head)
+
+    return bitarray(b)
+
+
 def huffman_code(freq_map, endian=None):
     """huffman_code(dict, /, endian=None) -> dict
 
@@ -264,7 +312,7 @@ hashable object (including `None`).
     import heapq
 
     if not isinstance(freq_map, dict):
-        raise TypeError("dict expected")
+        raise TypeError("dict expected, got '%s'" % type(freq_map).__name__)
     if len(freq_map) == 0:
         raise ValueError("non-empty dict expected")
     if endian is None:
